
import random

from .. import api

from .snake import Snake


class StarterSnake(Snake):
    '''Snake from Battlesnake's starter snake repo'''

    name = "starter_snake"

    def info(self) -> api.GetResponse:
        print("INFO")
        return api.GetResponse(
            apiversion="1",
            author="shillerben",
            color="#BBDEFB",
            head="default",
            tail="default",
        )

    def start(self, game_state: api.StartRequest) -> api.StartResponse:
        print("GAME START")
        return api.StartResponse()

    def end(self, game_state: api.EndRequest) -> api.EndResponse:
        print("GAME OVER")
        return api.EndResponse()

    def move(self, game_state: api.MoveRequest) -> api.MoveResponse:
        is_move_safe = {"up": True, "down": True, "left": True, "right": True}

        # We've included code to prevent your Battlesnake from moving backwards
        my_head = game_state.you.body[0]  # Coordinates of your head
        my_neck = game_state.you.body[1]  # Coordinates of your "neck"

        if my_neck.x < my_head.x:  # Neck is left of head, don't move left
            is_move_safe["left"] = False

        elif my_neck.x > my_head.x:  # Neck is right of head, don't move right
            is_move_safe["right"] = False

        elif my_neck.y < my_head.y:  # Neck is below head, don't move down
            is_move_safe["down"] = False

        elif my_neck.y > my_head.y:  # Neck is above head, don't move up
            is_move_safe["up"] = False

        # TODO: Step 1 - Prevent your Battlesnake from moving out of bounds
        board_width = game_state.board.width
        board_height = game_state.board.height
        if my_head.x == 0:
            is_move_safe["left"] = False
        elif my_head.x == board_width - 1:
            is_move_safe["right"] = False
        if my_head.y == 0:
            is_move_safe["down"] = False
        elif my_head.y == board_height - 1:
            is_move_safe["up"] = False

        # TODO: Step 2 - Prevent your Battlesnake from colliding with itself
        # my_body = game_state['you']['body']

        # TODO: Step 3 - Prevent your Battlesnake from colliding with other Battlesnakes
        # opponents = game_state['board']['snakes']

        # Are there any safe moves left?
        safe_moves = []
        for move, isSafe in is_move_safe.items():
            if isSafe:
                safe_moves.append(move)

        if len(safe_moves) == 0:
            print(f"MOVE {game_state.turn}: No safe moves detected! Moving down")
            return api.MoveResponse(move=api.GameMove.UP)

        # Choose a random move from the safe ones
        next_move = random.choice(safe_moves)

        # TODO: Step 4 - Move towards food instead of random, to regain health and survive longer
        # food = game_state['board']['food']

        print(f"MOVE {game_state.turn}: {next_move}")
        return api.MoveResponse(move=api.GameMove(next_move))
